import * as Y from "yjs";
import { MissionMap } from "../storage/missionMap";
import MissionService from "../service/operation/missionService";
import { UpdateMissionCommand } from "../command/operation/mission/missionCommand";
import MissionCommandHandler from "../command/operation/mission/missionCommandHandler";

export default abstract class MissionDocHelper {
  public static async populateDoc(missionId: string) {
    const mission = await MissionService.getFullById(missionId);

    const doc = new Y.Doc();
    doc.getMap("form").set("title", mission.title);
    doc.getMap("form").set("command", mission.commandId ?? "");
    doc.getMap("form").set("secretary", mission.secretaryId ?? "");
    doc.getMap("form").set("start", mission.mission_start?.toISOString() ?? "");
    doc.getMap("form").set("end", mission.mission_end?.toISOString() ?? "");
    doc.getMap("form").set("mission_short", mission.keyword);
    doc.getMap("form").set("location", mission.location);
    doc.getMap("form").set("others", mission.others);
    doc.getMap("form").set("rescued", mission.rescued.toString());
    doc.getMap("form").set("recovered", mission.recovered.toString());
    doc.getText("editor").applyDelta(JSON.parse(mission.description)); //.insert(0, mission.description);
    for (const vehicle of mission.vehicles) {
      doc.getMap<Y.Map<number | string>>("vehicle").set(vehicle.vehicleId, new Y.Map<string | number>());
      for (const key of ["driverId", "leaderId", "mileage_start", "mileage_end"] as (keyof typeof vehicle)[]) {
        doc
          .getMap<Y.Map<number | string>>("vehicle")
          .get(vehicle.vehicleId)
          .set<string | number>(key, vehicle[key].toString());
      }
    }
    for (const equipment of mission.equipments) {
      doc.getMap<Y.Map<number | string>>("equipment").set(equipment.equipmentId, new Y.Map<string | number>());
      for (const key of ["note"] as (keyof typeof equipment)[]) {
        doc
          .getMap<Y.Map<number | string>>("equipment")
          .get(equipment.equipmentId)
          .set<string | number>(key, equipment[key].toString());
      }
    }
    for (const contact of mission.contacts) {
      doc.getMap<Y.Map<number | string>>("contact").set(contact.contactId, new Y.Map<string | number>());
      for (const key of ["firstname", "lastname", "phone", "address", "note"] as (keyof typeof contact)[]) {
        doc
          .getMap<Y.Map<number | string>>("contact")
          .get(contact.contactId)
          .set<string | number>(key, contact[key].toString());
      }
    }
    for (const force of mission.forces) {
      doc.getMap<boolean>("presence").set(force.forceId, true);
    }

    MissionMap.write(missionId, {
      missionId,
      doc,
      editors: new Map(),
      editorStates: new Map(),
      timestamp: 0,
    });
    console.log(`created local doc ${missionId}`);
  }

  public static docAsUpdate(missionId: string): Uint8Array {
    const mission = MissionMap.read(missionId);
    return Y.encodeStateAsUpdate(mission.doc);
  }

  public static docAsStateVector(missionId: string): Uint8Array {
    const mission = MissionMap.read(missionId);
    return Y.encodeStateVector(mission.doc);
  }

  public static docTimestamp(missionId: string): number {
    const mission = MissionMap.read(missionId);
    return mission.timestamp;
  }

  public static getAwarenessState(missionId: string) {
    const mission = MissionMap.read(missionId);
    return Array.from(mission.editorStates.entries());
  }

  public static getAwarenessEditors(missionId: string) {
    const mission = MissionMap.read(missionId);
    return Array.from(mission.editors.entries());
  }

  public static async saveDoc(missionId: string, update: Uint8Array) {
    const oldMission = MissionMap.read(missionId);
    const oldVehicles = Array.from(oldMission.doc.getMap("presence").keys());
    const oldEquipments = Array.from(oldMission.doc.getMap("presence").keys());
    const oldContact = Array.from(oldMission.doc.getMap("presence").keys());
    const oldPresence = Array.from(oldMission.doc.getMap("presence").keys());

    MissionMap.updateState(missionId, update);

    const mission = MissionMap.read(missionId);
    const vehicles = Array.from(mission.doc.getMap("presence").keys());
    const equipments = Array.from(mission.doc.getMap("presence").keys());
    const contact = Array.from(mission.doc.getMap("presence").keys());
    const presence = Array.from(mission.doc.getMap("presence").keys());

    const removedVehicles = oldVehicles.filter((item) => !vehicles.includes(item));
    const removedEquipments = oldEquipments.filter((item) => !equipments.includes(item));
    const removedContacts = oldContact.filter((item) => !contact.includes(item));
    const removedPresence = oldPresence.filter((item) => !presence.includes(item));

    //editor.setContents(mission.doc.getText("editor").toDelta()); //new QuillDeltaToHtmlConverter(mission.doc.getText("editor").toDelta()).convert(),
    const missionDetails: UpdateMissionCommand = {
      id: missionId,
      title: mission.doc.getMap("form").toJSON().title,
      ...(mission.doc.getMap("form").toJSON().command != ""
        ? { commandId: mission.doc.getMap("form").toJSON().command }
        : { commandId: null }),
      ...(mission.doc.getMap("form").toJSON().secretary != ""
        ? { secretaryId: mission.doc.getMap("form").toJSON().secretary }
        : { secretaryId: null }),
      ...(mission.doc.getMap("form").get("start") != ""
        ? { mission_start: new Date(mission.doc.getMap("form").get("start") as string) }
        : { mission_start: null }),
      ...(mission.doc.getMap("form").get("end") != ""
        ? { mission_end: new Date(mission.doc.getMap("form").get("end") as string) }
        : { mission_end: null }),
      keyword: mission.doc.getMap("form").toJSON().mission_short,
      location: mission.doc.getMap("form").toJSON().location,
      others: mission.doc.getMap("form").toJSON().others,
      rescued: parseInt(mission.doc.getMap("form").toJSON().rescued),
      recovered: parseInt(mission.doc.getMap("form").toJSON().recovered),
      description: JSON.stringify(mission.doc.getText("editor").toDelta()),
    };
    const missionVehicles = mission.doc.getMap<Y.Map<string | number>>("vehicle").toJSON();
    const missionEquipment = mission.doc.getMap<Y.Map<string | number>>("equipment").toJSON();
    const missionContact = mission.doc.getMap<Y.Map<string | number>>("contact").toJSON();
    const missionPresence = mission.doc.getMap<Y.Map<string | number>>("presence").toJSON();

    // store Data to database

    // Object.keys(tmp).map(t => ({
    //   id:t,
    //   ...tmp[t]
    // }))

    await MissionCommandHandler.sync(missionDetails);
  }
}
